//
//  GameLibraryView.swift
//  NoDiceAlpha
//
//  Created by Ben Albright on 4/5/24.
//

import SwiftUI
import Foundation
import SDWebImageSwiftUI
import Firebase
import FirebaseStorage
import FirebaseFirestore

func fetchBoardGameInfo(for id: Int) -> BoardGameInfo? {
    let url = URL(string: "https://api.geekdo.com/xmlapi/boardgame/\(id)")!
    do {
        let data = try Data(contentsOf: url)
        let parser = XMLParser(data: data)
        let delegate = BoardGameInfoXMLParserDelegate()
        parser.delegate = delegate
        parser.parse()
        return delegate.boardGameInfo
    } catch {
        print("Error fetching board game info: \(error.localizedDescription)")
        return nil
    }
}
struct BoardGameInfo {
    var gameName: String
    var minPlayers: Int
    var maxPlayers: Int
    var minPlaytime: Int
    var maxPlaytime: Int
    var thumbnail: String
    var image: String
    var description: String
    var publishedYear: String
}
struct BoardGame: Identifiable, Decodable {
    
    let id: Int
    var name: String
    var info: BoardGameInfo?
    enum CodingKeys: String, CodingKey {
        case id
        case name
    }
    
    init(from decoder: Decoder) throws {
        let container = try decoder.container(keyedBy: CodingKeys.self)
        id = try container.decode(Int.self, forKey: .id)
        name = try container.decode(String.self, forKey: .name)
        info = nil
    }
}
class BoardGameInfoXMLParserDelegate: NSObject, XMLParserDelegate {
    
  var boardGameInfo: BoardGameInfo? = BoardGameInfo(gameName: "", minPlayers: 0, maxPlayers: 0, minPlaytime: 0, maxPlaytime: 0, thumbnail: "", image: "", description: "", publishedYear: "")
  var currentElement: String = ""
  var currentValue: String = ""
  func parser(_ parser: XMLParser, didStartElement elementName: String, namespaceURI: String?, qualifiedName qName: String?, attributes attributeDict: [String : String] = [:]) {
    currentElement = elementName
    currentValue = ""
  }
   
  func parser(_ parser: XMLParser, foundCharacters string: String) {
    currentValue += string
  }
   
  func parser(_ parser: XMLParser, didEndElement elementName: String, namespaceURI: String?, qualifiedName qName: String?) {
    switch elementName {
    case "name":
      boardGameInfo?.gameName = currentValue
    case "minplayers":
        boardGameInfo?.minPlayers = Int(currentValue) ?? 0 
    case "maxplayers":
        boardGameInfo?.maxPlayers = Int(currentValue) ?? 0
    case "minplaytime":
        boardGameInfo?.minPlaytime = Int(currentValue) ?? 0
    case "maxplaytime":
        boardGameInfo?.maxPlaytime = Int(currentValue) ?? 0
    case "thumbnail":
        boardGameInfo?.thumbnail = currentValue
    case "image":
        boardGameInfo?.image = currentValue
    case "description":
        boardGameInfo?.description = currentValue
    case "yearpublished":
        boardGameInfo?.publishedYear = currentValue
    default:
      break
    }
    currentValue = ""
  }
}
class BoardGamesLoader: ObservableObject {
    @Published var boardGames = [BoardGame]()
    init() {
        if let url = Bundle.main.url(forResource: "boardgames_ranks", withExtension: "json") {
            do {
                let data = try Data(contentsOf: url)
                let decoder = JSONDecoder()
                self.boardGames = try decoder.decode([BoardGame].self, from: data)
            } catch {
                print("Error loading JSON: \(error.localizedDescription)")
            }
        }
    }
}

struct GameLibraryView: View {
    
    @ObservedObject var boardGamesLoader = BoardGamesLoader()
    @State private var searchText = ""
    @State private var filteredBoardGames: [BoardGame] = []
    @State private var showingAlert = false
    @State private var currentPage = 1
    @State var isLoading: Bool = false
    @State var errorMessage: String = ""
    @State var showError: Bool = false
    
    var body: some View {
        NavigationView {
            VStack {
                Text("Games must be in your collection if you want use them in posts.")
                    .font(.caption2)
                    .foregroundColor(.gray)
                    .padding(.top, 3)
                    .hAlign(.leading)
                    .padding(.horizontal)
                
                SearchBar(text: $searchText, onSearchTapped: search)
                
                List(filteredBoardGames) { boardGame in
                    GameListItemView(game: boardGame)
                }
                .alert(isPresented: $showingAlert) {
                    Alert(title: Text("No Results"), message: Text("No board games found matching your search."), dismissButton: .default(Text("OK")))
                }
                .overlay(content: {
                    LoadingView(show: $isLoading)
                })
                //if filteredBoardGames.count >= 25 {
                    //HStack {
                        //Spacer()
                        //Button("Load More") {
                            //loadMoreGames()
                        //}
                        //.padding()
                    //}
                //}
            }
            .navigationBarTitle("Game Library")
        }
        .onAppear {
            loadInitialGames()
        }
    }
    
    func loadMoreGames() {
        print("Loading more games...")
        let startIndex = currentPage * 10
            let endIndex = min(startIndex + 10, filteredBoardGames.count)
            let slicedGames = filteredBoardGames[startIndex..<endIndex]
          
            for game in slicedGames {
                if let info = fetchBoardGameInfo(for: game.id) {
                    var game = game
                    game.info = info
                    filteredBoardGames.append(game)
                } else {
              print("Error fetching info for game \(game.id)")
                }
          }
    }
    
    func search() {
        //if searchText.isEmpty {
            //filteredBoardGames = Array(boardGamesLoader.boardGames.shuffled().prefix(10))
            //return
        //}
        
        isLoading = true
        closeKeyboard()
        
        Task{
            do{
                var allFetchedGames: [BoardGame] = []
                let filteredGames = boardGamesLoader.boardGames.filter { $0.name.lowercased().contains(searchText.lowercased()) }
                allFetchedGames.append(contentsOf: filteredGames)
                filteredBoardGames = allFetchedGames.filter { $0.name.lowercased().contains(searchText.lowercased()) }
                if filteredGames.isEmpty {
                    showingAlert = true
                }
                
                _ = (currentPage - 1) * 10
                filteredBoardGames = filteredGames.prefix(10).map { boardGame in
                    var game = boardGame
                    if let info = fetchBoardGameInfo(for: boardGame.id) {
                        game.info = info
                    } else {
                        print("Error fetching info for game \(game.id)")
                    }
                    return game
                }
            }
        }
        DispatchQueue.main.async {
            isLoading = false
        }
    }
    
    func loadInitialGames() {
        isLoading = true
        
        Task{
            do{
                let initialGames = Array(boardGamesLoader.boardGames.prefix(300))
                
                let shuffledGames = initialGames.shuffled()
                
                filteredBoardGames = shuffledGames.prefix(10).map { boardGame in
                    var game = boardGame
                    if let info = fetchBoardGameInfo(for: boardGame.id) {
                        game.info = info
                    } else {
                        print("Error fetching info for game \(game.id)")
                    }
                    return game
                }
            }
        }
        DispatchQueue.main.async {
            isLoading = false
        }
    }
    
    func setError(_ error: Error)async {
        await MainActor.run(body: {
            isLoading = false
            errorMessage = error.localizedDescription
            showError.toggle()
        })
    }
}

struct SearchBar: View {
    @Binding var text: String
    var onSearchTapped: () -> Void
    var body: some View {
        HStack {
            TextField("Search", text: $text)
                .padding(7)
                .background(Color(.white))
                .cornerRadius(8)
                .padding(.horizontal, 10)
                .padding(.vertical, 5)
            
            Button(action: {
                onSearchTapped()
            }) {
                Text("Search")
                    .padding(.horizontal, 10)
                    .padding(.vertical, 5)
                    .background(Color("nd-purple"))
                    .foregroundColor(.white)
                    .cornerRadius(8)
            }
            .padding(.trailing, 10)
        }
    }
}

struct GameListItemView: View {
    
    let game: BoardGame
    @State private var thumbnailImage: UIImage? = nil
    
    var body: some View {
        NavigationLink(destination: GameDetailView(game: game)) {
            HStack {
                if let thumbnailImage = thumbnailImage {
                    Image(uiImage: thumbnailImage)
                        .resizable()
                        .aspectRatio(contentMode: .fit)
                        .frame(width: 50, height: 50)
                        .padding(.trailing, 10)
                        .cornerRadius(5)
                }
                VStack(alignment: .leading) {
                    Text("\(game.name) (\(game.info?.publishedYear ?? ""))")
                        .font(.headline)
                        .fontWeight(.bold)
                    HStack {
                        
                        if game.info?.minPlayers == game.info?.maxPlayers {
                            Text("\(game.info?.minPlayers ?? 0) Players")
                        } else {
                            Text("\(game.info?.minPlayers ?? 0)-\(game.info?.maxPlayers ?? 0) Players")
                        }
                        
                        Spacer()
                        
                        if game.info?.minPlaytime == game.info?.maxPlaytime {
                            Text("\(game.info?.minPlaytime ?? 0) min")
                        } else {
                            Text("\(game.info?.minPlaytime ?? 0)-\(game.info?.maxPlaytime ?? 0) min")
                        }
                        
                        Spacer()
                    }
                    .font(.subheadline)
                    .foregroundColor(.gray)
                }
            }
            .onAppear {
                if let imageUrlString = game.info?.thumbnail,
                   let url = URL(string: imageUrlString) {
                    DispatchQueue.global().async {
                        do {
                            let imageData = try Data(contentsOf: url)
                            if let uiImage = UIImage(data: imageData) {
                                DispatchQueue.main.async {
                                    self.thumbnailImage = uiImage
                                }
                            }
                        } catch {
                            print("Error loading image: \(error)")
                        }
                    }
                }
            }
        }
    }
}

struct GameDetailView: View {
    
    @State private var isSaved: Bool = false
    let game: BoardGame
    var body: some View {
        
        VStack {
            if let imageUrlString = game.info?.image,
               let url = URL(string: imageUrlString),
               let imageData = try? Data(contentsOf: url),
               let uiImage = UIImage(data: imageData) {
                Image(uiImage: uiImage)
                    .resizable()
                    .aspectRatio(contentMode: .fit)
                    .frame(width: 200, height: 200)
                    .padding(.top, -50)
            }
            HStack {
                Text("\(game.name) (\(game.info?.publishedYear ?? ""))")
                    .font(.title)
                    .fontWeight(.bold)
                    .padding(.horizontal)
                    .padding(.bottom)
                    .padding(.top)
                Spacer()
            }
            
            ScrollView(showsIndicators: true) {
                
            HStack {
                if game.info?.minPlayers == game.info?.maxPlayers {
                    Text("Players: \(game.info?.minPlayers ?? 0)")
                } else {
                    Text("Players: \(game.info?.minPlayers ?? 0)-\(game.info?.maxPlayers ?? 0)")
                }
                Spacer()
            }
            .font(.title3)
            .fontWeight(.semibold)
            .padding(.horizontal)
            .padding(.bottom)
            
            HStack {
                if game.info?.minPlaytime == game.info?.maxPlaytime {
                    Text("Play Time: \(game.info?.minPlaytime ?? 0) min")
                } else {
                    Text("Play Time: \(game.info?.minPlaytime ?? 0)-\(game.info?.maxPlaytime ?? 0) min")
                }
                
                Spacer()
            }
            .font(.title3)
            .fontWeight(.semibold)
            .padding(.horizontal)
            .padding(.bottom)
            
            Text("\(processDescription(game.info?.description ?? ""))")
                .padding(.horizontal)
            }
            
            HStack{
                if isSaved == false{
                    Button("Add to Collection"){
                        addToCollection()
                    }
                    .padding()
                    .background(Color("nd-purple"))
                    .clipShape(Capsule())
                    .foregroundColor(.white)
                    .font(.headline)
                    .fontWeight(.bold)
                }else{
                    Text("Game Saved!")
                        .padding()
                        .background(Color("nd-purple"))
                        .clipShape(Capsule())
                        .foregroundColor(.white)
                        .font(.headline)
                        .fontWeight(.bold)
                }
                
                //Button("Use in a Post"){
                    //
                //}
                //.padding()
                //.background(Color("nd-green"))
                //.clipShape(Capsule())
                //.foregroundColor(.black)
                //.font(.headline)
                //.fontWeight(.bold)
            }
            .padding(.top, 2)
            .hAlign(.center)
            
        }
        .multilineTextAlignment(.leading)
        .padding()
    }
    
    
    func processDescription(_ description: String) -> String {
        return description.replacingOccurrences(of: "<br/>", with: "\n")
            .replacingOccurrences(of: "&mdash;", with: "-")
            .replacingOccurrences(of: "&quot;", with: "'")
            .replacingOccurrences(of: "&amp;", with: "&")
            .replacingOccurrences(of: "&ndash;", with: "-")
            .replacingOccurrences(of: "&nbsp;", with: " ")

    }
    
    func addToCollection() {
        guard let userID = Auth.auth().currentUser?.uid else{return}
        let savedGame = UserSavedGame(game: game, userID: userID)
        saveGameToFirestore(savedGame: savedGame)
        isSaved.toggle()
      }
    
    func saveGameToFirestore(savedGame: UserSavedGame) {
        let db = Firestore.firestore()
        let userID = savedGame.userID

        db.collection("Users").document(userID).collection("savedGames").document(savedGame.id).setData([
            "name": savedGame.name,
            "id": savedGame.id,
            "thumbnailURL": savedGame.thumbnailURL,
            "minPlayers": savedGame.minPlayers,
            "maxPlayers": savedGame.maxPlayers,
            "minPlaytime": savedGame.minPlaytime,
            "maxPlaytime": savedGame.maxPlaytime,
            "image": savedGame.image,
            "description": savedGame.description,
            "publishedYear": savedGame.publishedYear,
            "userID": userID
        ]) { error in
            if let error = error {
                print(error.localizedDescription)
            } else {
                print("Game Saved Successfully")
            }
        }
    }

}

#Preview {
    GameLibraryView()
}
