//
//  PostCardView.swift
//  NoDiceAlpha
//
//  Created by Ben Albright on 4/8/24.
//

import SwiftUI
import SDWebImageSwiftUI
import Firebase
import FirebaseStorage

struct PostCardView: View {
    
    var post: Post
    
    var onUpdate: (Post)->()
    var onDelete: ()->()
    
    @AppStorage("user_UID") private var userUID: String = ""
    @State private var docListener: ListenerRegistration?
    
    var body: some View {
        HStack(alignment: .top, spacing: 12){
            WebImage(url: post.userProfileURL)
                .resizable()
                .aspectRatio(contentMode: .fill)
                .frame(width: 45, height: 45)
                .clipShape(Circle())
            
            VStack(alignment: .leading, spacing: 6){
                Text(post.userName)
                    .font(.callout)
                    .fontWeight(.semibold)
                    .padding(.top,12)
                    .padding(.bottom, 12)
                
                Divider()
                
                Text(post.gameNameSpelled)
                    .font(.title3)
                    .fontWeight(.semibold)
                    .padding(.bottom, -6)
                    .padding(.top, 4)
                    .lineLimit(nil)
                
                HStack{
                    VStack{
                        Image("star\(post.rating)")
                            .resizable()
                            .scaledToFit()
                            .frame(width: 125)
                            .padding(.bottom, -12)
                            .padding(.top, 3)
                            .hAlign(.leading)
                        
                        Text("**Players**: \(post.playersPicked)")
                            .font(.callout)
                            .foregroundColor(.black)
                            .padding(.vertical)
                            .hAlign(.leading)
                        
                        Text("**Duration**: \(post.timePicked)")
                            .font(.callout)
                            .foregroundColor(.black)
                            .padding(.bottom)
                            .hAlign(.leading)
                            .lineLimit(3)

                    }
                    .hAlign(.leading)
                    
                    Spacer()
                    
                    AsyncImage(url: URL(string: post.imageReferenceID)) { phase in
                        if let image = phase.image {
                            image
                                .resizable()
                                .scaledToFit()
                                .frame(width: 100, height: 100)
                                .padding(.horizontal)
                                .padding(.vertical)
                        } else if phase.error != nil {
                            Text("Error loading image")
                        } else {
                            ProgressView()
                        }
                    }
                }
                .hAlign(.leading)
            
                Text("**Review**: \(post.text)")
                    .textSelection(.enabled)
                    .padding(.vertical, 8)
                    .padding(.bottom, 10)
                    .padding(.trailing, 8)
                    .padding(.top, -9)
                    .lineLimit(nil)
                    .fixedSize(horizontal: false, vertical: true)
                
                HStack{
                    PostInteraction()
                    
                    Spacer()
                    
                    Text(post.publishedDate.formatted(date: .numeric, time: .omitted))
                        .font(.caption2)
                        .foregroundColor(.gray)
                }
                .padding(.bottom, 12)
            }
        }
        .hAlign(.leading)
        .overlay(alignment: .topTrailing, content: {
            if post.userUID == userUID{
                Menu{
                    Button("Delete Post",role: .destructive, action: deletePost)
                }label: {
                    Image(systemName: "ellipsis")
                        .font(.caption)
                        .rotationEffect(.init(degrees: -90))
                        .foregroundColor(.gray)
                        .padding(6)
                        .contentShape(Rectangle())
                }
                .offset(x: 6)
            }
        })
        .onAppear {
            if docListener == nil {
                guard let postID = post.id else{return}
                docListener = Firestore.firestore().collection("Posts").document(postID).addSnapshotListener({ snapshot, error in
                    if let snapshot{
                        if snapshot.exists{
                            if let updatedPost = try? snapshot.data(as: Post.self) {
                                onUpdate(updatedPost)
                            }
                        }else{
                            onDelete()
                        }
                    }
                })
            }
        }
        .onDisappear {
            if let docListener{
                docListener.remove()
                self.docListener = nil
            }
        }
    }
    
    @ViewBuilder
    func PostInteraction()-> some View{
        HStack(spacing: 6){
            Button(action: likePost){
                Image(systemName: post.likedIDs.contains(userUID) ? "flame.fill" : "flame")
                    .scaleEffect(1.5)
            }
            
            Text("\(post.likedIDs.count)")
                .font(.footnote)
                .foregroundColor(.gray)
            
        }
    }
    
    func likePost(){
        Task{
            guard let postID = post.id else{return}
            if post.likedIDs.contains(userUID){
                try await Firestore.firestore().collection("Posts").document(postID).updateData([
                    "likedIDs": FieldValue.arrayRemove([userUID])
                ])
            }else{
                try await Firestore.firestore().collection("Posts").document(postID).updateData([
                    "likedIDs": FieldValue.arrayUnion([userUID])
                ])
            }
        }
    }
    
    func deletePost(){
        Task{
            do{
                guard let postID = post.id else{return}
                try await Firestore.firestore().collection("Posts").document(postID).delete()
            }catch{
                print(error.localizedDescription)
            }
        }
    }
}

//#Preview {
//    PostCardView()
//}
