//
//  ReusableProfileContent.swift
//  NoDiceAlpha
//
//  Created by Ben Albright on 4/5/24.
//

import SwiftUI
import Foundation
import SDWebImage
import SDWebImageSwiftUI
import Firebase
import FirebaseFirestore
import FirebaseStorage

class UserViewModel: ObservableObject {
  @Published var savedGames: [UserSavedGame] = []
  
  func loadSavedGames( for userID: String) {
    FirestoreManager.shared.getSavedGames(for: userID) { savedGames in
      self.savedGames = savedGames
    }
  }
}

class FirestoreManager {
  static let shared = FirestoreManager()
  private let db = Firestore.firestore()

  func getSavedGames( for userID: String, completion: @escaping ([UserSavedGame]) -> Void) {
    db.collection("Users").document(userID).collection("savedGames")
      .getDocuments { (querySnapshot, error) in
        if let error = error {
          print(error.localizedDescription)
          completion([])
          return
        }
        
        guard let documents = querySnapshot?.documents else {
          completion([])
          return
        }

        let savedGames = documents.compactMap { document -> UserSavedGame? in
          return try? document.data(as: UserSavedGame.self)
        }
        completion(savedGames)
      }
  }
}

struct sampleBoardGame{
    var name: String
    var players: String
    var duration: String
    var publisher: String
    var pubYear: String
    var image: String
    var description: String
}


struct ReusableProfileContent: View {
    
    @ObservedObject var viewModel: UserViewModel
    let userID: String
    var user: User
    @State private var fetchedPosts: [Post] = []
    @State private var viewMode: ViewMode = .posts
    @State private var savedGames: [UserSavedGame] = []
    
    enum ViewMode {
      case posts
      case savedGames
    }
    
    var body: some View {
        ScrollView(.vertical, showsIndicators: false){
            LazyVStack{
                HStack(spacing: 12){
                    WebImage(url: user.userProfileURL) { image in
                        image.resizable()
                    } placeholder: {
                        Image("NullProfile")
                            .resizable()
                    }
                    .aspectRatio(contentMode: .fill)
                    .frame(width: 100, height: 100)
                    .clipShape(Circle())
                    
                    VStack(alignment: .leading, spacing: 6){
                        Text(user.username)
                            .font(.title3)
                            .fontWeight(.bold)
                        
                        Text(user.userBio)
                            .font(.caption)
                            .foregroundColor(.gray)
                            .lineLimit(3)
                        
                        if let bioLink = URL(string: user.userBioLink) {
                            Link(user.userBioLink, destination: bioLink)
                                .font(.callout)
                                .tint(Color("nd-purple"))
                                .lineLimit(1)
                        }
                    }
                    .hAlign(.leading)
                }
                .padding()
                
                if viewMode == .posts {
                    
                    HStack{
                        Button{
                            viewMode = .posts
                        }label: {
                            HStack{
                                Text("Posts")
                                    .font(.title2)
                                    .fontWeight(.bold)
                                    .foregroundColor(.black)
                                    //.hAlign(.leading)
                                
                                Image(systemName: "rectangle.portrait.on.rectangle.portrait.angled.fill")
                                    .font(.title2)
                                    .foregroundColor(Color("nd-purple"))
                            }
                            .padding()
                            .hAlign(.leading)
                        }
                        
                        Button{
                            viewMode = .savedGames
                        }label: {
                            HStack{
                                Text("Games")
                                    .font(.title2)
                                    .fontWeight(.bold)
                                    .foregroundColor(.gray)
                                    //.hAlign(.leading)
                                
                                Image(systemName: "books.vertical.fill")
                                    .font(.title2)
                                    .foregroundColor(.gray)
                            }
                            .padding()
                            .hAlign(.leading)
                        }
                        
                    }
                    //ProfileGamesView(viewModel: SavedGamesViewModel(), userID: user.userUID)
                    ReusablePostsView(basedOnUID: true, uid: user.userUID, posts: $fetchedPosts)
                    
                }else{
                    
                        HStack{
                            Button{
                                viewMode = .posts
                            }label: {
                                HStack{
                                    Text("Posts")
                                        .font(.title2)
                                        .fontWeight(.bold)
                                        .foregroundColor(.gray)
                                    //.hAlign(.leading)
                                    
                                    Image(systemName: "rectangle.portrait.on.rectangle.portrait.angled.fill")
                                        .font(.title2)
                                        .foregroundColor(.gray)
                                }
                                .padding()
                                .hAlign(.leading)
                            }
                            
                            Button{
                                viewMode = .savedGames
                            }label: {
                                HStack{
                                    Text("Games")
                                        .font(.title2)
                                        .fontWeight(.bold)
                                        .foregroundColor(.black)
                                    
                                    Image(systemName: "books.vertical.fill")
                                        .font(.title2)
                                        .foregroundColor(Color("nd-green"))
                                }
                                .padding()
                                .hAlign(.leading)
                            }
                        }
                    ScrollView{
                        NavigationView{
                            List {
                                ForEach(viewModel.savedGames) { savedGame in
                                    SavedGameListItemView(game: savedGame)
                                }
                            }
                            .onAppear {
                                viewModel.loadSavedGames(for: userID)
                            }
                        }
                        .edgesIgnoringSafeArea(.bottom)
                }
                    
                    //List {
                    //    ForEach(viewModel.savedGames) { savedGame in
                            //SavedGameListItemView(game: savedGame)
                    //        Text(savedGame.name)
                    //    }
                    //}
                    //.onAppear {
                    //      viewModel.loadSavedGames(for: userID)
                    //}
                    
                    //ForEach(gameList.indices, id: \.self) { index in
                    //    HStack{
                    //        Image("\(gameList[index].image)")
                    //            .resizable()
                    //            .aspectRatio(contentMode: .fit)
                    //            .frame(width: 50, height: 50)
                    //            .padding(.trailing, 10)
                    //            .cornerRadius(5)
                            
                    //        VStack{
                    //            Text("\(gameList[index].name) (\(gameList[index].pubYear))")
                    //                .font(.headline)
                    //                .fontWeight(.bold)
                    //                .hAlign(.leading)
                                
                    //            HStack{
                    //                Text(gameList[index].players)
                    //
                    //                Spacer()
                                    
                    //                Text(gameList[index].duration)
                                    
                    //                Spacer()
                    //            }
                    //            .font(.subheadline)
                    //            .foregroundColor(.gray)
                    //        }
                    //        .hAlign(.leading)
                    //    }
                    //}
                }
            }
        }
    }
    
    func fetchGamesOld()async{
        do{
            var query: Query!
            query = Firestore.firestore().collection("savedGames")
                .order(by: "name", descending: true)
            
            let docs = try await query.getDocuments()
            let fetchedGames = docs.documents.compactMap { doc -> UserSavedGame? in
                try? doc.data(as: UserSavedGame.self)
            }
            await MainActor.run(body: {
                savedGames.append(contentsOf: fetchedGames)
                //isFetching = false
            })
        }catch{
            print(error.localizedDescription)
        }
    }
}
